package at.ac.uibk.gitsearch.domain.util;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class TreeNode<T> {

    private final T data;
    private final TreeNode<T> parent;
    private final List<TreeNode<T>> children = new ArrayList<>();

    public interface TreeCollector<S, T> {
        S collect(TreeNode<T> tree);
    }

    public TreeNode(T nodeData) {
        data = nodeData;
        parent = null;
    }

    public TreeNode(T nodeData, TreeNode<T> parent) {
        data = nodeData;
        this.parent = parent;
    }

    public void addChild(T nodeData) {
        TreeNode<T> node = new TreeNode<>(nodeData, this);
        addChild(node);
    }

    public void addChild(TreeNode<T> nodeData) {
        children.add(nodeData);
    }

    public void addChildren(List<TreeNode<T>> nodeData) {
        for (TreeNode<T> node : nodeData) {
            children.add(node);
        }
    }

    public T getData() {
        return data;
    }

    public TreeNode<T> getParent() {
        return parent;
    }

    public List<TreeNode<T>> getChildren() {
        return children;
    }

    /**
     * visits all nodes depths forst and collects the results
     *
     * @param <S>
     * @param collector
     * @return
     */
    public <S> List<S> collect(TreeCollector<S, T> collector) {
        final Stream<List<S>> mappedChildren = children.stream().map(node -> node.collect(collector));
        List<S> result = mappedChildren.flatMap(List::stream).collect(Collectors.toList());
        result.add(collector.collect(this));
        return result;
    }
}
