import { Component, OnInit } from '@angular/core';
import { NgbActiveModal } from '@ng-bootstrap/ng-bootstrap';
import { AlertService } from 'app/core/util/alert.service';
import { GitlabGroup } from 'app/shared/model/gitlab-group.model';
import { GitlabService } from 'app/shared/gitlab/gitlab.service';

@Component({
  selector: 'jhi-gitlab-path-selector',
  templateUrl: './gitlab-path-selector.component.html',
  styleUrls: ['./gitlab-path-selector.component.scss'],
})
export class GitlabPathSelectorComponent implements OnInit {
  groups: GitlabGroup[] | undefined;
  subgroups: GitlabGroup[] | undefined;

  isLoading = false;
  errorOccurred = false;

  constructor(private activeModal: NgbActiveModal, private alertService: AlertService, private gitlabService: GitlabService) {}

  ngOnInit(): void {
    this.isLoading = true;
    this.gitlabService.getGroups().subscribe({
      next: data => {
        this.groups = data;
        this.isLoading = false;
      },
      error: error => {
        this.alertService.addAlert({
          type: 'danger',
          translationKey: error.error,
          //              translationParams: { param: alertParams },
        });
        this.isLoading = false;
        this.errorOccurred = true;
      },
    });
  }

  /**
   * Utility function used to retrieve the upper groups in the
   * hierarchy
   */
  getParentGroups(): GitlabGroup[] | undefined {
    return this.groups?.filter(group => !group.parentId);
  }

  /**
   * Utility function used to retrieve the subgroups of
   * a given group
   * @param group to search for subGroups
   */
  getSubGroupsOfGroup(group: GitlabGroup): GitlabGroup[] | undefined {
    return this.groups?.filter(subGroup => subGroup.parentId === group.id);
  }

  closeModal(group: GitlabGroup | undefined): void {
    this.activeModal.close(group);
  }
}
