package at.ac.uibk.gitsearch.security.oauth2;

import static org.assertj.core.api.Assertions.assertThat;

import at.ac.uibk.gitsearch.security.oauth2.UserDetailsFetcher.UserDetailsInfo;
import com.google.common.collect.Lists;
import java.util.List;
import java.util.Optional;
import org.gitlab4j.api.models.ImpersonationToken;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.junit.jupiter.api.Test;

class UserDetailsFetcherTest {

    @Test
    void testNormalizationWithRealExample() {
        UserDetailsFetcher udf = new UserDetailsFetcher();
        final List<String> realExample = Lists.newArrayList(
            "sharing/university-innsbruck/c-exercises/eidp-21s",
            "general1",
            "sharing/graz-university-of-technology",
            "internal",
            "development",
            "sharing/university-innsbruck/python/general",
            "sharing/university-innsbruck/java",
            "sharing/university-klagenfurt",
            "sharing/university-salzburg",
            "sharing/johannes-kepler-university-linz",
            "sharing",
            "sharing/danube-university-krems",
            "development/sharing",
            "sharing/university-innsbruck/python",
            "sharing/vienna-university-of-technology",
            "ROLE_USER",
            "sharing/university-innsbruck/c-exercises",
            "sharing/university-innsbruck/modelling",
            "sharing/university-innsbruck",
            "sharing/johannes-kepler-university-linz/java",
            "ROLE_ADMIN",
            "sharing/university-innsbruck/java/general"
        );
        MatcherAssert.assertThat(
            udf.reduceGroups(realExample),
            Matchers.containsInAnyOrder("general1", "internal", "development", "sharing", "ROLE_USER", "ROLE_ADMIN")
        );
    }

    @Test
    void testNormalizationWithComplexExample() {
        UserDetailsFetcher udf = new UserDetailsFetcher();
        final List<String> complexExample = Lists.newArrayList(
            "sharing/university-innsbruck/c-exercises/eidp-21s",
            "sharing",
            "sharing/university-innsbruck",
            "sharing/university-innsbruck/java/general",
            "sharing2/university-innsbruck/java/general"
        );
        MatcherAssert.assertThat(
            udf.reduceGroups(complexExample),
            Matchers.containsInAnyOrder("sharing", "sharing2/university-innsbruck/java/general")
        );
    }

    @Test
    void testUserDetailsInfo() {
        UserDetailsInfo udi = new UserDetailsInfo(Optional.empty(), false);

        assertThat(udi.getUserToken()).isEmpty();
        assertThat(udi.isModified()).isFalse();

        ImpersonationToken t = new ImpersonationToken();

        udi.setUserToken(Optional.of(t));
        udi.setModified(true);

        assertThat(udi.getUserToken()).contains(t);
        assertThat(udi.isModified()).isTrue();
    }
}
